@@ -5,10 +5,16 @@ mod utils;
 pub use self::utils::*;
 use super::*;
 use crate::{
-    syntax_error, systems::borders::TailwindBorderStyle, TailwindBorderCollapse, TailwindBoxDecorationBreak, TailwindBoxSizing,
-    TailwindClear, TailwindColumns, TailwindContainer, TailwindDisplay, TailwindFloat, TailwindFontFamily, TailwindFontSize,
-    TailwindFontSmoothing, TailwindFontWeight, TailwindHeight, TailwindIsolation, TailwindPosition, TailwindScreenReader,
-    TailwindSpacing, TailwindTextAlignment, TailwindTextColor, TailwindVisibility, TailwindWidth,
+    syntax_error,
+    systems::{
+        borders::TailwindBorderStyle,
+        filters::TailwindBrightness,
+        flexbox::{TailWindFlexGrow, TailWindFlexShrink, TailWindOrder, TailwindFlex, TailwindFlexDirection, TailwindFlexWrap},
+    },
+    TailwindBorderCollapse, TailwindBoxDecorationBreak, TailwindBoxSizing, TailwindClear, TailwindColumns, TailwindContainer,
+    TailwindDisplay, TailwindFloat, TailwindFontFamily, TailwindFontSize, TailwindFontSmoothing, TailwindFontWeight,
+    TailwindIsolation, TailwindPosition, TailwindScreenReader, TailwindSizing, TailwindSpacing, TailwindTextAlignment,
+    TailwindTextColor, TailwindVisibility,
 };
 use std::{
     fmt::{Display, Formatter, Write},
@@ -85,9 +91,6 @@ impl Display for AstStyle {
 }
 
 impl TailwindInstance for AstStyle {
-    fn selectors(&self, ctx: &TailwindBuilder) -> String {
-        todo!()
-    }
     fn attributes(&self, ctx: &TailwindBuilder) -> BTreeSet<CssAttribute> {
         let mut out = BTreeSet::default();
         match self.get_instance() {
@@ -101,37 +104,39 @@ impl TailwindInstance for AstStyle {
 }
 // noinspection SpellCheckingInspection
 impl AstStyle {
+    #[inline(never)]
     pub fn get_instance(&self) -> Result<Box<dyn TailwindInstance>> {
-        let instance = match self.view_elements().as_slice() {
+        let element = self.view_elements();
+        let arbitrary = self.view_arbitrary();
+        let instance = match element.as_slice() {
             // ["w", rest @ ..] => {TailW}
             // Layout System
-            ["aspect", rest @ ..] => TailwindAspect::parse(rest)?.boxed(),
+            ["aspect", rest @ ..] => TailwindAspect::parse(rest, arbitrary)?.boxed(),
             ["container"] => TailwindContainer {}.boxed(),
             ["columns", rest @ ..] => TailwindColumns::parse(rest)?.boxed(),
             ["break", "before", rest @ ..] => TailwindBreak::parse_before(rest)?.boxed(),
             ["break", "inside", rest @ ..] => TailwindBreak::parse_inside(rest)?.boxed(),
             ["break", "after", rest @ ..] => TailwindBreak::parse_after(rest)?.boxed(),
-            ["box", rest @ ..] => Self::box_adaptor(rest)?,
+            ["box", rest @ ..] => Self::box_adaptor(rest, arbitrary)?,
             ["block"] => TailwindDisplay::Block.boxed(),
-            ["flex"] => TailwindDisplay::Flex.boxed(),
             // begin https://tailwindcss.com/docs/display
             ["inline", "flex"] => TailwindDisplay::InlineFlex.boxed(),
             ["inline", "block"] => TailwindDisplay::InlineBlock.boxed(),
             // end https://tailwindcss.com/docs/display
-            ["float", rest @ ..] => Self::float_adaptor(rest)?,
+            ["float", rest @ ..] => Self::float_adaptor(rest, arbitrary)?,
             ["clear", rest @ ..] => TailwindClear::parse(rest)?.boxed(),
 
             ["isolate"] => TailwindIsolation::Isolate.boxed(),
             ["isolation", "auto"] => TailwindIsolation::Auto.boxed(),
-            ["object", rest @ ..] => Self::object_adaptor(rest)?,
-            ["overflow", rest @ ..] => Self::overflow_adaptor(rest)?,
-            ["overscroll", rest @ ..] => Self::overscroll_adaptor(rest)?,
+            ["object", rest @ ..] => Self::object_adaptor(rest, arbitrary)?,
+            ["overflow", rest @ ..] => Self::overflow_adaptor(rest, arbitrary)?,
+            ["overscroll", rest @ ..] => Self::overscroll_adaptor(rest, arbitrary)?,
             // begin https://tailwindcss.com/docs/position
-            ["static"] => TailwindPosition::InlineFlex.boxed(),
-            ["fixed"] => TailwindPosition::InlineFlex.boxed(),
-            ["absolute"] => TailwindPosition::InlineFlex.boxed(),
-            ["relative"] => TailwindPosition::InlineFlex.boxed(),
-            ["sticky"] => TailwindPosition::InlineFlex.boxed(),
+            ["static"] => TailwindPosition::Static.boxed(),
+            ["fixed"] => TailwindPosition::Fixed.boxed(),
+            ["absolute"] => TailwindPosition::Absolute.boxed(),
+            ["relative"] => TailwindPosition::Relative.boxed(),
+            ["sticky"] => TailwindPosition::Sticky.boxed(),
             // end https://tailwindcss.com/docs/position
             ["visible"] => TailwindVisibility::Visible.boxed(),
             ["invisible"] => TailwindVisibility::Invisible.boxed(),
@@ -139,25 +144,35 @@ impl AstStyle {
             ["inset", rest @ ..] => TailWindZIndex::parse(rest, self.negative),
 
             ["s", rest @ ..] => TailWindZIndex::parse(rest, self.negative),
+            // Flexbox & Grid
+            ["flex", rest @ ..] => Self::flex_adaptor(rest, arbitrary)?,
+            ["grow", rest @ ..] => TailWindFlexGrow::parse(rest, arbitrary)?.boxed(),
+            ["shrink", rest @ ..] => TailWindFlexShrink::parse(rest, arbitrary)?.boxed(),
+            ["order", rest @ ..] => TailWindOrder::parse(rest, arbitrary, self.negative)?.boxed(),
+
             // Spacing System
-            [p @ ("p" | "pl" | "pr" | "pm" | "pt" | "px" | "py"), rest @ ..] => TailwindSpacing::parse_padding(rest, p),
-            [m @ ("m" | "ml" | "mr" | "mm" | "mt" | "mx" | "my"), rest @ ..] => TailwindSpacing::parse_margin(rest, m),
-            ["space", "x", rest @ ..] => TailwindSpacing::parse_space(rest, 'x'),
-            ["space", "y", rest @ ..] => TailwindSpacing::parse_space(rest, 'y'),
+            [p @ ("p" | "pl" | "pr" | "pm" | "pt" | "px" | "py"), rest @ ..] => {
+                TailwindSpacing::parse_padding(rest, p, arbitrary)?.boxed()
+            }
+            [m @ ("m" | "ml" | "mr" | "mm" | "mt" | "mx" | "my"), rest @ ..] => {
+                TailwindSpacing::parse_margin(rest, m, arbitrary)?.boxed()
+            }
+            ["space", "x", rest @ ..] => TailwindSpacing::parse_space(rest, 'x', arbitrary)?.boxed(),
+            ["space", "y", rest @ ..] => TailwindSpacing::parse_space(rest, 'y', arbitrary)?.boxed(),
             // Sizing System
-            ["w", rest @ ..] => TailwindWidth::parse(rest, "normal"),
-            ["min", "w", rest @ ..] => TailwindWidth::parse(rest, "min"),
-            ["max", "w", rest @ ..] => TailwindWidth::parse(rest, "max"),
-            ["h", rest @ ..] => TailwindHeight::parse(rest, "normal"),
-            ["min", "h", rest @ ..] => TailwindHeight::parse(rest, "min"),
-            ["max", "h", rest @ ..] => TailwindHeight::parse(rest, "max"),
+            ["w", rest @ ..] => TailwindSizing::parse_width(rest, arbitrary)?.boxed(),
+            ["min", "w", rest @ ..] => TailwindSizing::parse_width_min(rest, arbitrary)?.boxed(),
+            ["max", "w", rest @ ..] => TailwindSizing::parse_width_max(rest, arbitrary)?.boxed(),
+            ["h", rest @ ..] => TailwindSizing::parse_width(rest, arbitrary)?.boxed(),
+            ["min", "h", rest @ ..] => TailwindSizing::parse_width_min(rest, arbitrary)?.boxed(),
+            ["max", "h", rest @ ..] => TailwindSizing::parse_width_max(rest, arbitrary)?.boxed(),
             // Typography System
-            ["font", rest @ ..] => Self::font_adaptor(rest)?,
+            ["font", rest @ ..] => Self::font_adaptor(rest, arbitrary)?,
             // begin https://tailwindcss.com/docs/font-variant-numeric
             ["normal", "nums"] => todo!(),
             ["ordinal"] => todo!(),
             // end https://tailwindcss.com/docs/font-variant-numeric
-            ["text", rest @ ..] => Self::text_adaptor(rest)?,
+            ["text", rest @ ..] => Self::text_adaptor(rest, arbitrary)?,
             ["antialiased"] => TailwindFontSmoothing::new(false).boxed(),
             ["subpixel", "antialiased"] => TailwindFontSmoothing::new(true).boxed(),
             ["italic"] => todo!(),
@@ -165,7 +180,7 @@ impl AstStyle {
             // TODO:https://tailwindcss.com/docs/font-variant-numeric
             ["tracking", rest @ ..] => todo!(),
             ["leading", rest @ ..] => todo!(),
-            ["list", rest @ ..] => Self::list_adaptor(rest)?,
+            ["list", rest @ ..] => Self::list_adaptor(rest, arbitrary)?,
             ["text", alignment @ ("text" | "serif" | "mono")] => todo!(),
             // TODO:https://tailwindcss.com/docs/font-variant-numeric
             ["underline", rest @ ..] => todo!(),
@@ -181,16 +196,16 @@ impl AstStyle {
             // Borders System
             ["rounded", rest @ ..] => todo!(),
             ["border", rest @ ..] => Self::border_adaptor(rest)?,
-            ["divide", rest @ ..] => Self::divide_adaptor(rest)?,
-            ["outline", rest @ ..] => Self::outline_adaptor(rest)?,
-            ["ring", rest @ ..] => Self::ring_adaptor(rest)?,
+            ["divide", rest @ ..] => Self::divide_adaptor(rest, arbitrary)?,
+            ["outline", rest @ ..] => Self::outline_adaptor(rest, arbitrary)?,
+            ["ring", rest @ ..] => Self::ring_adaptor(rest, arbitrary)?,
             // Effects System
-            ["shadow", rest @ ..] => Self::shadow_adaptor(rest)?,
+            ["shadow", rest @ ..] => Self::shadow_adaptor(rest, arbitrary)?,
             ["opacity", rest @ ..] => todo!(),
             ["mix", "blend", rest @ ..] => todo!(),
             // Filters System
             ["blur", rest @ ..] => todo!(),
-            ["brightness", rest @ ..] => todo!(),
+            ["brightness", rest @ ..] => TailwindBrightness::parse(rest, arbitrary)?.boxed(),
             ["contrast", rest @ ..] => todo!(),
 
             ["drop", "shadow", rest @ ..] => todo!(),
@@ -198,7 +213,7 @@ impl AstStyle {
             ["grayscale", rest @ ..] => todo!(),
             ["backdrop", rest @ ..] => todo!(),
             // Tables System
-            ["table", rest @ ..] => Self::table_adaptor(rest)?,
+            ["table", rest @ ..] => Self::table_adaptor(rest, arbitrary)?,
             // Transitions System
             ["transition", rest @ ..] => todo!(),
             ["duration", rest @ ..] => todo!(),
@@ -227,7 +242,7 @@ impl AstStyle {
             ["sr", "only"] => TailwindScreenReader::new(true).boxed(),
             ["not", "sr", "only"] => TailwindScreenReader::new(false).boxed(),
             // Form System Extension
-            _ => panic!("Unknown tailwind system"),
+            _ => return syntax_error!("Unknown instructions: {} + [{}]", element.join("-"), arbitrary),
         };
         Ok(instance)
     }
@@ -255,65 +270,118 @@ impl AstStyle {
             ["black"] => TailwindBorderCollapse::Separate.into_instance(),
             ["white"] => TailwindBorderCollapse::Separate.into_instance(),
 
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown border instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn divide_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn divide_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
+        let out = match str {
+            // https://tailwindcss.com/docs/divide-width
+            ["x"] => todo!(),
+            ["x", n] => todo!(),
+            ["y"] => todo!(),
+            ["y", n] => todo!(),
+            // https://tailwindcss.com/docs/divide-style
+            ["solid"] => todo!(),
+            ["dashed"] => todo!(),
+            ["dotted"] => todo!(),
+            ["double"] => todo!(),
+            ["none"] => todo!(),
+            // https://tailwindcss.com/docs/divide-color
+            _ => return syntax_error!("Unknown divide instructions: {}", str.join("-")),
+        };
+        Ok(out)
+    }
+    #[inline]
+    fn outline_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
+        let out = match str {
+            // https://tailwindcss.com/docs/outline-width
+            [n] => todo!(),
+            // https://tailwindcss.com/docs/outline-style
+            [] => todo!(),
+            ["none"] => todo!(),
+            ["dashed"] => todo!(),
+            ["dotted"] => todo!(),
+            ["double"] => todo!(),
+            ["hidden"] => todo!(),
+            // https://tailwindcss.com/docs/outline-offset
+            ["offset", n] => todo!(),
+            _ => return syntax_error!("Unknown outline instructions: {}", str.join("-")),
+        };
+        Ok(out)
+    }
+    #[inline]
+    fn ring_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/float
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown ring instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn outline_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn shadow_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
-            // https://tailwindcss.com/docs/float
-            _ => return syntax_error!(""),
+            // https://tailwindcss.com/docs/box-shadow
+            ["sm"] => todo!(),
+            // https://tailwindcss.com/docs/box-shadow-color
+            _ => return syntax_error!("Unknown shadow instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn ring_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
-        let out = match str {
-            // https://tailwindcss.com/docs/float
-            _ => return syntax_error!(""),
-        };
-        Ok(out)
-    }
-    #[inline]
-    fn shadow_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
-        let out = match str {
-            // https://tailwindcss.com/docs/float
-            _ => return syntax_error!(""),
-        };
-        Ok(out)
-    }
-    #[inline]
-    fn box_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn box_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             ["decoration", "clone"] => TailwindBoxDecorationBreak::Clone.boxed(),
             ["decoration", "slice"] => TailwindBoxDecorationBreak::Clone.boxed(),
             ["border"] => TailwindBoxSizing::Border.boxed(),
             ["content"] => TailwindBoxSizing::Content.boxed(),
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown box instructions: {}", str.join("-")),
+        };
+        Ok(out)
+    }
+    #[inline]
+    fn flex_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
+        let out = match str {
+            // https://tailwindcss.com/docs/display#flex
+            [] if arbitrary.is_empty() => TailwindDisplay::Flex.boxed(),
+            // https://tailwindcss.com/docs/flex#arbitrary-values
+            [] => TailwindFlex::parse_arbitrary(arbitrary)?.boxed(),
+            // https://tailwindcss.com/docs/flex-direction
+            ["row"] => TailwindFlexDirection::Row.boxed(),
+            ["row", "reverse"] => TailwindFlexDirection::RowReverse.boxed(),
+            ["col"] => TailwindFlexDirection::Column.boxed(),
+            ["col", "reverse"] => TailwindFlexDirection::ColumnReverse.boxed(),
+            // https://tailwindcss.com/docs/flex-wrap
+            ["wrap"] => TailwindFlexWrap::Wrap.boxed(),
+            ["wrap", "reverse"] => TailwindFlexWrap::WrapReverse.boxed(),
+            ["nowrap"] => TailwindFlexWrap::NoWrap.boxed(),
+            // https://tailwindcss.com/docs/flex
+            ["auto"] => TailwindBoxDecorationBreak::Clone.boxed(),
+            ["initial"] => TailwindBoxDecorationBreak::Clone.boxed(),
+            ["none"] => TailwindBoxDecorationBreak::Clone.boxed(),
+            [n] => TailwindFlex::parse(n)?.boxed(),
+            _ => return syntax_error!("Unknown box instructions: {}", str.join("-")),
         };
         Ok(out)
     }
 
     #[inline]
-    fn list_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn list_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/list-style-type
             ["none"] => todo!(),
-            _ => return syntax_error!(""),
+            ["disc"] => todo!(),
+            ["decimal"] => todo!(),
+            // https://tailwindcss.com/docs/list-style-position
+            ["inside"] => todo!(),
+            ["outside"] => todo!(),
+            _ => return syntax_error!("Unknown list instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn table_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn table_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/display#table
             [] => TailwindDisplay::Table.boxed(),
@@ -323,50 +391,49 @@ impl AstStyle {
             // https://tailwindcss.com/docs/table-layout
             ["auto"] => TailwindTableLayout::Auto.boxed(),
             ["fixed"] => TailwindTableLayout::Fixed.boxed(),
-
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown table instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn float_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn float_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/float
             ["left"] => TailwindFloat::Left.boxed(),
             ["right"] => TailwindFloat::Right.boxed(),
             ["none"] => TailwindFloat::None.boxed(),
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown float instructions: {}", str.join("-")),
         };
         Ok(out)
     }
 
     #[inline]
-    fn object_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn object_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/float
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown object instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn overflow_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn overflow_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/float
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown overflow instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn overscroll_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn overscroll_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/float
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown overscroll instructions: {}", str.join("-")),
         };
         Ok(out)
     }
 
     #[inline]
-    fn font_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn font_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/float
             ["sans"] => TailwindFontFamily::Sans.boxed(),
@@ -396,12 +463,12 @@ impl AstStyle {
             ["bold"] => TailwindFontWeight::BOLD.boxed(),
             ["extrabold"] => TailwindFontWeight::EXTRA_BOLD.boxed(),
             ["black"] => TailwindFontWeight::BLACK.boxed(),
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown font instructions: {}", str.join("-")),
         };
         Ok(out)
     }
     #[inline]
-    fn text_adaptor(str: &[&str]) -> Result<Box<dyn TailwindInstance>> {
+    fn text_adaptor(str: &[&str], arbitrary: &str) -> Result<Box<dyn TailwindInstance>> {
         let out = match str {
             // https://tailwindcss.com/docs/text-align
             ["left"] => TailwindTextAlignment::Left.boxed(),
@@ -415,15 +482,8 @@ impl AstStyle {
             ["black"] => TailwindTextColor::BLACK.boxed(),
             ["white"] => TailwindTextColor::WHITE.boxed(),
             [color, weight] => TailwindTextColor::parse(color, weight)?.boxed(),
-            _ => return syntax_error!(""),
+            _ => return syntax_error!("Unknown text instructions: {}", str.join("-")),
         };
         Ok(out)
     }
 }
-
-impl TailwindBuilder {
-    /// `(item (WS/NL item)*)?`
-    pub(crate) fn parse(&self, input: &str) -> Result<BTreeSet<Box<dyn TailwindInstance>>> {
-        todo!()
-    }
-}
