@@ -2,14 +2,27 @@ use super::*;
 
 impl Display for TailwindAspect {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        write!(f, "aspect-{}", self.kind)
+        match self {
+            Self::Auto => {
+                write!(f, "aspect-auto")
+            }
+            Self::Arbitrary(a, b) => {
+                write!(f, "aspect-{}/{}", a, b)
+            }
+        }
     }
 }
 
 impl TailwindInstance for TailwindAspect {
     fn attributes(&self, _: &TailwindBuilder) -> BTreeSet<CssAttribute> {
+        let ar = match self {
+            Self::Auto => "auto",
+            Self::Arbitrary(_, _) => {
+                todo!()
+            }
+        };
         css_attributes! {
-            "aspect-ratio" => self.ratio
+            "aspect-ratio" => ar
         }
     }
 }
